searchData={"items":[{"type":"module","title":"ConfigSmuggler","doc":"ConfigSmuggler is a library for converting Elixir-style configuration\nstatements to and from string-encoded key/value pairs.\n\nElixir (and Erlang)'s configuration system is somewhat richer than\nnaïve environment variables, i.e., `System.get_env/1`-style key/value\nconfigs alone can capture.\n\nConfigs in Elixir are namespaced by app, can be arbitrarily nested, and\ncontain Elixir-native data types like atoms, keyword lists, etc.\n\nConfigSmuggler provides a bridge between Elixir applications and\nkey/value configuration stores, especially those available at runtime.\nIt makes it dead-simple to use platform-agnostic configuration\nsystems with Elixir services.","ref":"ConfigSmuggler.html"},{"type":"module","title":"WARNING! - ConfigSmuggler","doc":"The functions in `ConfigSmuggler` are *not suitable* for use on\nuntrusted inputs!  Code is `eval`ed, atoms are created, etc.\n\nConfigs are considered privileged inputs, so don't worry about using\nConfigSmuggler for its intended purpose.  But please, *never* let user\ninput anywhere near this module.  You've been warned.","ref":"ConfigSmuggler.html#module-warning"},{"type":"module","title":"Example - ConfigSmuggler","doc":"iex> encoded_configs = %{\n    ...>   # imagine you fetch this every 60 seconds at runtime\n    ...>   \"elixir-logger-level\" => \":debug\",\n    ...>   \"elixir-my_api-MyApi.Endpoint-url-port\" => \"8888\",\n    ...> }\n    iex> ConfigSmuggler.apply(encoded_configs)\n    iex> Application.get_env(:logger, :level)\n    :debug\n    iex> ConfigSmuggler.encode([my_api: Application.get_all_env(:my_api)])\n    {:ok, %{\"elixir-my_api-MyApi.Endpoint-url-port\" => \"8888\"}}","ref":"ConfigSmuggler.html#module-example"},{"type":"module","title":"Overview - ConfigSmuggler","doc":"* `apply/1` applies encoded or decoded configs to the current environment.\n\n* `decode/1` converts an encoded config map into Elixir-native decoded\n  configs, also returning a list of zero or more encoded key/value pairs\n  that could not be decoded.\n\n* `encode/1` converts Elixir-native decoded configs\n  (i.e., a keyword list with app name as key and keyword list of\n  configs as value) into an encoded config map.\n\n* `encode_file/1` converts an entire `config.exs`-style file\n  (along with all files included with `Mix.Config.import_config/1`)\n  into an encoded config map.\n\n* `encode_statement/1` converts a single `config` statement from a\n  `config.exs`-style file into an encoded config map.\n\n* At the command line, `mix smuggle encode  ` encodes\n  a file with `encode_file/1` and emits a JSON object as output.","ref":"ConfigSmuggler.html#module-overview"},{"type":"module","title":"Encoding Scheme - ConfigSmuggler","doc":"The encoded key begins with `elixir` and is a hyphen-separated \"path\"\nof atoms and modules leading to the config value we wish to set.\n\nThe value is any valid Elixir term, encoded using normal Elixir syntax.\n\nEncoding is performed by `Kernel.inspect/2`.\nDecoding is performed by `Code.eval_string/1` and `String.to_atom/1`.","ref":"ConfigSmuggler.html#module-encoding-scheme"},{"type":"module","title":"See Also - ConfigSmuggler","doc":"If you build and deploy Erlang releases, and you want to apply encoded\nconfigs before any other apps have started, look into [Distillery\nconfig providers](https://hexdocs.pm/distillery/config/runtime.html#config-providers).\n\nThis feature allows specified modules to make environment changes\nwith `Application.put_env/3`, after which these changes are persisted to\nthe release's `sys.config` file and the release is started normally.","ref":"ConfigSmuggler.html#module-see-also"},{"type":"module","title":"Gotchas - ConfigSmuggler","doc":"Atoms and modules are expected to follow standard Elixir convention,\nnamely that atoms begin with a lowercase letter, modules begin with\nan uppercase letter, and neither contains any hyphen characters.\n\nIf a config file or statement makes reference to `Mix.env()`, the current\nMix env will be substituted.  This may be different than what the config\nfile intended.","ref":"ConfigSmuggler.html#module-gotchas"},{"type":"module","title":"Authorship and License - ConfigSmuggler","doc":"Copyright 2019, [Appcues, Inc.](https://www.appcues.com)\n\nConfigSmuggler is released under the [MIT\nLicense](https://github.com/appcues/config_smuggler/blob/master/MIT_LICENSE.txt).","ref":"ConfigSmuggler.html#module-authorship-and-license"},{"type":"function","title":"ConfigSmuggler.apply/1","doc":"Applies the given config to the current environment (i.e., calls\n`Application.put_env/3` a bunch of times).  Accepts Elixir-\nnative decoded configs or encoded config maps.\n\n    iex> ConfigSmuggler.apply([my_app: [foo: 22]])\n    iex> Application.get_env(:my_app, :foo)\n    22\n\n    iex> ConfigSmuggler.apply(%{\"elixir-my_app-bar\" => \"33\"})\n    iex> Application.get_env(:my_app, :bar)\n    33","ref":"ConfigSmuggler.html#apply/1"},{"type":"function","title":"ConfigSmuggler.decode/1","doc":"Decodes a map of string-encoded key/value pairs into a keyword list of\nElixir configs, keyed by app.  Also returns a list of zero or more invalid\nkey/value pairs along with their errors.\n\n    iex> ConfigSmuggler.decode(%{\n    ...>   \"elixir-my_app-some_key\" => \"22\",\n    ...>   \"elixir-my_app-MyApp.Endpoint-url-host\" => \"\\\"localhost\\\"\",\n    ...>   \"elixir-logger-level\" => \":info\",\n    ...>   \"elixir-my_app-MyApp.Endpoint-url-port\" => \"4444\",\n    ...>   \"bad key\" => \"22\",\n    ...>   \"elixir-my_app-foo\" => \"bogus value\",\n    ...> })\n    {:ok,\n      [\n        my_app: [\n          {:some_key, 22},\n          {MyApp.Endpoint, [\n            url: [\n              port: 4444,\n              host: \"localhost\",\n            ]\n          ]},\n        ],\n        logger: [\n          level: :info,\n        ],\n      ],\n      [\n        {{\"elixir-my_app-foo\", \"bogus value\"}, :bad_value},\n        {{\"bad key\", \"22\"}, :bad_key},\n      ]\n    }","ref":"ConfigSmuggler.html#decode/1"},{"type":"type","title":"ConfigSmuggler.decoded_configs/0","doc":"","ref":"ConfigSmuggler.html#t:decoded_configs/0"},{"type":"function","title":"ConfigSmuggler.encode/1","doc":"Converts Elixir-native decoded configs (i.e., a keyword list with\napp name as key and keyword list of configs as value) into an\nencoded config map.\n\n    iex> ConfigSmuggler.encode([logger: [level: :info], my_app: [key: \"value\"]])\n    {:ok, %{\n        \"elixir-logger-level\" => \":info\",\n        \"elixir-my_app-key\" => \"\\\"value\\\"\",\n    }}","ref":"ConfigSmuggler.html#encode/1"},{"type":"function","title":"ConfigSmuggler.encode_file/1","doc":"Reads a config file and returns a map of encoded key/value pairs\nrepresenting the configuration.  Respects `Mix.Config.import_config/1`.\n\n    iex> ConfigSmuggler.encode_file(\"config/config.exs\")\n    {:ok, %{\n      \"elixir-logger-level\" => \":info\",\n      # ...\n    }}","ref":"ConfigSmuggler.html#encode_file/1"},{"type":"function","title":"ConfigSmuggler.encode_statement/1","doc":"Encodes a single `Mix.Config.config/2` or `Mix.Config.config/3`\nstatement into one or more encoded key/value pairs.\n\n    iex> ConfigSmuggler.encode_statement(\"config :my_app, key1: :value1, key2: \\\"value2\\\"\")\n    {:ok, %{\n        \"elixir-my_app-key1\" => \":value1\",\n        \"elixir-my_app-key2\" => \"\\\"value2\\\"\",\n    }}\n\n    iex> ConfigSmuggler.encode_statement(\"config :my_app, MyApp.Endpoint, url: [host: \\\"localhost\\\", port: 4444]\")\n    {:ok, %{\n      \"elixir-my_app-MyApp.Endpoint-url-host\" => \"\\\"localhost\\\"\",\n      \"elixir-my_app-MyApp.Endpoint-url-port\" => \"4444\",\n    }}","ref":"ConfigSmuggler.html#encode_statement/1"},{"type":"type","title":"ConfigSmuggler.encoded_config_map/0","doc":"","ref":"ConfigSmuggler.html#t:encoded_config_map/0"},{"type":"type","title":"ConfigSmuggler.encoded_key/0","doc":"","ref":"ConfigSmuggler.html#t:encoded_key/0"},{"type":"type","title":"ConfigSmuggler.encoded_value/0","doc":"","ref":"ConfigSmuggler.html#t:encoded_value/0"},{"type":"type","title":"ConfigSmuggler.error_reason/0","doc":"","ref":"ConfigSmuggler.html#t:error_reason/0"},{"type":"type","title":"ConfigSmuggler.validation_error/0","doc":"","ref":"ConfigSmuggler.html#t:validation_error/0"},{"type":"task","title":"mix smuggle","doc":"ConfigSmuggler provides the `mix smuggle` task, which encodes\n`config.exs`-style files into JSON-formatted key/value pairs.\n\nUsage: `mix smuggle encode  `","ref":"Mix.Tasks.Smuggle.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}